/* SegorKat - A viewer for the electronic catalog of  
*            Segor electronics (www.segor.de). 
* 
* http://www.mydarc.de/dl9obn/programs/segorkat 
* 
* Copyright (C) 2005 by Dirk Baechle (dl9obn@darc.de) 
* 
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation; either version 2 
* of the License, or (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public 
* License along with this program; if not, write to the 
* 
* Free Software Foundation, Inc. 
* 675 Mass Ave 
* Cambridge 
* MA 02139 
* USA 
* 
*/

/** 
\file skSegorArticle.cpp 
\author Dirk Baechle 
\date 2005-07-02 
\version 1.0 
*/

/*------------------------------------------------------------- Includes */

#include "skGlobal.h"
#include "skSegorArticle.h"
#include "skFileIO.h"

/*------------------------------------------------------ Class functions */

/** Default constructor for the \a skSegorArticle class.
*/
skSegorArticle::skSegorArticle() : 
        parentID(),
	Name(),
        Description(),
	OtherNames(),
        Status(0),
        UnitPriceKey(0),
        AVTaxKey(0),
        PerUnit(0),
        OrderedUnits(0)
{
  ;
}

/** Copy constructor for the \a skSegorArticle class.
* @param elem Article info that is copied
*/
skSegorArticle::skSegorArticle(const skSegorArticle &elem) 
{
  parentID = elem.parentID;
  Name = elem.Name;
  Description = elem.Description;
  OtherNames = elem.OtherNames;
  Status = elem.Status;
  UnitPriceKey = elem.UnitPriceKey;
  AVTaxKey = elem.AVTaxKey;
  PerUnit = elem.PerUnit;
  OrderedUnits = elem.OrderedUnits;

  // Copy prices
  plPrices.clear();
  vector<skSegorPrice>::const_iterator p_it = elem.plPrices.begin();
  for (; p_it != elem.plPrices.end(); ++p_it)
    plPrices.push_back(*p_it);

}

/** Overloaded copy operator ``=''.
* @param elem Article info that is copied
*/
skSegorArticle &skSegorArticle::operator= (const skSegorArticle &elem)
{
  if (this == &elem)
    return *this;

  parentID = elem.parentID;
  Name = elem.Name;
  Description = elem.Description;
  OtherNames = elem.OtherNames;
  Status = elem.Status;
  UnitPriceKey = elem.UnitPriceKey;
  AVTaxKey = elem.AVTaxKey;
  PerUnit = elem.PerUnit;
  OrderedUnits = elem.OrderedUnits;

  // Copy prices
  plPrices.clear();
  vector<skSegorPrice>::const_iterator p_it = elem.plPrices.begin();
  for (; p_it != elem.plPrices.end(); ++p_it)
    plPrices.push_back(*p_it);

  return *this;
}

/** Reads a single entry from the given file stream \a f.
* @param f File stream, the entry is read from
* @return OK if entry was read successfully, ERROR else
*/
int skSegorArticle::read(ifstream &f)
{
  char c;
  
  plPrices.clear();
  parentID = "";
  Name = "";
  OtherNames = "";
  Description = "";

  // Read ID
  int i;
  for (i = 0; i < 5; ++i)
  {
    f.get(c);
    parentID += c;
  }
  // Read name
  bool finished = false;
  for (i = 0; i < 24; ++i)
  {
    f.get(c);
    if ((0 != c) && (false == finished))
      Name += c;
    else
      finished = true;
  }
  simplifyWhiteSpace(Name);

  // Read other names for this item
  string singleName;
  int n;
  int added = 0;
  for (n = 0; n < 6; ++n)
  {
    singleName = "";
    finished = false;
    for (i = 0; i < 24; ++i)
    {
      f.get(c);
      if ((0 != c) && (false == finished))
        singleName += c;
      else
        finished = true;
    }
    simplifyWhiteSpace(singleName);
    if (singleName != "")
    {
      if (added > 0)
        OtherNames += "; ";
      OtherNames += singleName;
      ++added;
    }
  }
  simplifyWhiteSpace(OtherNames);

  // Read description
  finished = false;
  for (i = 0; i < 2*25; ++i)
  {
    f.get(c);
    if ((0 != c) && (false == finished))
      Description += c;
    else
      finished = true;
  }
  simplifyWhiteSpace(Description);
  // Read status
  Status = readInt(f);
  // Read items per unit
  PerUnit = readInt(f);
  // Read AVT key
  AVTaxKey = readInt(f);
  // Read unit key
  UnitPriceKey = readInt(f);

  // Read number of prices
  f.get(c);
  int pCnt = c;
  // Read prices
  skSegorPrice priceNew;
  for (i = 0; i < pCnt; ++i)
  {
    priceNew.readUnits(f);
    plPrices.push_back(priceNew);
  }
  vector<skSegorPrice>::iterator p_it = plPrices.begin();
  for (; p_it != plPrices.end(); ++p_it)
    p_it->readPrice(f);

  return OK;
}

/** Returns the minimum price that has to be paid
* when an article is ordered (=VPE*minUnit*minPrice).
*/
void skSegorArticle::getPriceInfo(int *vpe, int *minUnits, double *minPrice)
{
  if (plPrices.size() == 0)
  {
    *vpe = 0;
    *minUnits = 0;
    *minPrice = 0.0;
    return;
  }

  *minUnits = plPrices[0].Units;
  *vpe = PerUnit;
  *minPrice = (*minUnits) * (*vpe) * plPrices[0].Price;
  if (1 != UnitPriceKey)
  {
    *minPrice = (*minPrice)/100.0;
  }

}

/** Returns the current price sum for this article by taking into
* account the number of ordered units and the price table.
*/
void skSegorArticle::getCurrentPrice(double *nettoPrice, double *bruttoPrice)
{
  if (plPrices.size() == 0)
  {
    *nettoPrice = 0.0;
    *bruttoPrice = 0.0;
    return;
  }

  // Find appropriate price, depending on
  // the number of ordered units...
  vector<skSegorPrice>::iterator p_it = plPrices.begin();
  while ((p_it != plPrices.end()) &&
         (OrderedUnits >= p_it->Units))
    ++p_it;  

  // Step one back
  --p_it;

  *bruttoPrice = OrderedUnits * PerUnit * (p_it->Price);
  if (1 != UnitPriceKey)
  {
    *bruttoPrice = (*bruttoPrice)/100.0;
  }

  if (1 != AVTaxKey)
    *nettoPrice = *bruttoPrice / 1.07;
  else
    *nettoPrice = *bruttoPrice / 1.16;

}

